/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.report.NewReportFolderViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");
dojo.require("dojo.parser");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.PermissionsWidget");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");

var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var ReportMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var ReportClient			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var session					= com.ibm.team.repository.web.client.session;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var PermissionsWidget		= com.ibm.rational.report.web.core.app.internal.common.PermissionsWidget;

var FOLDER_CLOSED			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/folder_closed.gif");
var FOLDER_CLOSED_SHARED	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/folder_closed_shared.gif");
var tabClickEventHandles = [];

/**
 * Displays the New Report Folder view.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.report.NewReportFolderViewer",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/report/templates/NewReportFolderViewer.html"),
	
	startup: function() {
		this.inherited(arguments);
		
		// instantiate widgets used to display tabs
		var nodeList = [];
		nodeList.push(this.layoutContainer);
		nodeList.push(this.mainContentPane);
		nodeList.push(this.tabContainer);
		nodeList.push(this.detailsContentPane);
		if (this._createPermissionsTab) {
			nodeList.push(this.permissionContentPane);
		}
		var nodes = dojo.parser.instantiate(nodeList);
		
		var tabContainerId = nodes[2].id;
		this.tabContainerDomNode = nodes[2].domNode;
		dojo.subscribe(tabContainerId+"-selectChild", this, "_newTabSelected");
		
		var tabs = dojo.query(".dijitTabInnerDiv", this.tabContainerDomNode);		
		for (var i = 1; i < tabs.length; i++) {
			var handle = dojo.connect(tabs[i], "onclick", this, "_disableTabs");
			tabs[i].setAttribute("class", "dijitTabInnerDiv disabledTabs");
			tabClickEventHandles.push(handle);	
		}
		this.nodeShared.checked = this._shared;	//fix for IE6
		this._folderCreated = false;
	},
	
	constructor: function(args) {
		this._controller = args.controller;
		this._selectedParentNode = args.parentNode;
		this._createPermissionsTab = true;
		if (args.createPermissionsTab!=null) {
			this._createPermissionsTab = args.createPermissionsTab;
		}
		this._args = args;
	},
	
	postCreate: function() {
		this._initText();
						
		this._msgFormatter = new MessageFormatter(this.messageArea);
		this._postSave = null;
		this._postCancel = null;
		this._postServiceExec = null;		
		this._shared = null; 	// fix for IE6
		
		if (this._selectedParentNode != null) {
			this._selectedParentNodeId = this._selectedParentNode.itemId;
		}
		else {
			this._selectedParentNodeId = null;
		}
		
		var args = this._args;
		
		if (typeof args.postSave != "undefined") {
			this._postSave = args.postSave;
		}
		
		if (typeof args.postCancel != "undefined") {
			this._postCancel = args.postCancel;
		}
		
		var node = this._selectedParentNode;
		if (node != null) {			
			if (node.shared && !this._controller.secMgr.canShare(node)) {
				this._selectedParentNodeId = null;
			}
		}
				
		this._newTreeNode = null;
		this.permissionsWidget = null;		
		
		// fill the parent node control
		this._fillParentNodeControl();
		
		var self = this;
		
		dojo.connect(this.saveButton, "onclick",
			function(e) {
				e.preventDefault();
				self._handleSave();
			}
		);
		this.saveButton.disabled = true;
		
		dojo.connect(this.closeButton, "onclick",
			function(e) {
				e.preventDefault();
				if (self._postCancelFun) {
					self._postCancelFun();
				}
				else {
					self._controller.defaultAction();
				}
			}
		);
		
		dojo.connect(this.folderName, "onkeyup",
			function(e) {
				e.preventDefault();
				var name = dojo.trim(self.folderName.value);
				self.saveButton.disabled = (name.length == 0);
			}
		);
		
		dojo.connect(this.folderName, "onkeypress",
			function(e) {
				if(e.charCode == 47 || e.charCode == 92 || e.charCode == 42 || e.charCode == 63 ||
				   e.charCode == 124 || e.charCode == 62 || e.charCode == 60 || e.charCode == 58  || e.charCode == 34){
						e.preventDefault();
				}
			}
		);
			
		var postSave = function() {
			self._controller.newReportFolder(self._selectedParentNode, self._postSave, self._postCancel);
		};
		
		var postCancel = function() {
			self._controller.newReportFolder(self._selectedParentNode, self._postSave, self._postCancel);
		};
		
		dojo.connect(this.parentFolder, "onchange",
			function(e) {
				e.preventDefault();
				self._handleParentChange();
			}
		);		
		
		dojo.connect(this.createNode, "onkeypress",
			function(e) {
				if (e.keyCode == dojo.keys.ENTER) {
					e.preventDefault();
					self._handleSave();
				}
			}
		);
		
		dojo.connect(this.nodeShared, "onclick",
			function(e) {
				if (self._folderCreated == true && self.folderName.value.length > 0) {
					self.saveButton.disabled = (self.nodeShared.checked == self._shared);
				}
			}
		);
		
		if (this._createPermissionsTab == false) {
			this.tabContainer.removeChild(this.permissionContentPane);
		}
	},
	
	_initText: function() {
		Util.removeChildren(this.pageTitle);
		var pageTitleIcon = document.createElement("img");
		this.pageTitle.appendChild(pageTitleIcon);
		pageTitleIcon.src = FOLDER_CLOSED;		
		this.pageTitle.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_FOLDER_VIEWER_PAGE_TITLE));
		
		Util.removeChildren(this.saveButton);
		this.saveButton.appendChild(document.createTextNode(CommonMessages.SAVE_BUTTON_LABEL));
		
		Util.removeChildren(this.closeButton);
		this.closeButton.appendChild(document.createTextNode(CommonMessages.CLOSE_BUTTON_LABEL));
		
		Util.removeChildren(this.folderNameLabel);
		this.folderNameLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_FOLDER_VIEWER_NAME_LABEL));
			
		Util.removeChildren(this.parentFolderLabel);
		this.parentFolderLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_FOLDER_VIEWER_PARENT_FOLDER));
		
		Util.removeChildren(this.sharedLabel);
		this.sharedLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_FOLDER_VIEWER_SHARED_LABEL));
		
		this.detailsContentPane.title = ReportMessages.NEW_REPORT_FOLDER_VIEWER_DETAILS_TAB_TITLE;
		this.permissionContentPane.title = ReportMessages.NEW_REPORT_FOLDER_VIEWER_PERMISSIONS_TAB_TITLE;
	},	
	
	_fillParentNodeControl:  function() {
		this.parentFolder.options[0] = new Option(CommonMessages.NO_PARENT_FOLDER_OPTION, "", selected, selected);
		
		var parentNodes = this._controller.reportTreeParser.getValidParents(this._selectedNodeId);		
		if (parentNodes != null && parentNodes.length > 0) {
			for (var i = 0; i < parentNodes.length; i++) {
				var parentNode = parentNodes[i];
				
				if (parentNode.report) {
					continue;
				}
				
				var parentNodeId = parentNode.itemId;
				
				if (parentNodeId == "" || parentNodeId == null) {
					continue;
				}
				
				if (parentNodeId != this._selectedParentNodeId) {
					if (parentNode.shared && !this._controller.secMgr.canShare(parentNode)) {
						continue;
					}
				}
				
				var selected = (this._selectedParentNodeId != null) && (this._selectedParentNodeId == parentNodeId);				
				var opt = new Option(parentNode.name, parentNodeId, selected, selected);
				opt.node = parentNode;
				this.parentFolder.options[this.parentFolder.options.length] = opt;
				
				if (selected) {
					opt.selected = true; // fix for IE6
				}
			}
		}
		
		this._checkControlStates();	
	},
	
	_handleParentChange: function() {
		this._checkControlStates();
		if (this._folderCreated == true && this.folderName.value.length > 0) {
			var parentId = (this._selectedParentNodeId != null) ? this._selectedParentNodeId : "";
			if (parentId != this.parentFolder.options[this.parentFolder.selectedIndex].value) {
				this.saveButton.disabled = false;
			}
			else {
				this.saveButton.disabled = true;
			}
		}
	},
	
	_checkControlStates: function() {
		var shared = false;
		var disabled = false;
		
		if (!this._controller.secMgr.canShare()) {
			disabled = true;
		}
		
		if (this.parentFolder.selectedIndex >= 0) {
			var opt = this.parentFolder.options[this.parentFolder.selectedIndex];
			if (opt.node) {
				shared = opt.node.shared;
			}
			if (opt.value != "") {
				disabled = true;
			}
		}
		
		this.nodeShared.checked = shared;
		this._shared = shared;		//fix for IE6
		this.nodeShared.disabled = disabled;
	},
	
	_handleSave: function() {
		if (this._newTreeNode == null) {
			this._msgFormatter.displayNoMessage();
			
			var name = this.folderName.value;
  			for (var i = 0; i < name.length; i++) {
				if (name.charCodeAt(i) == 47 || name.charCodeAt(i) == 92 || name.charCodeAt(i) == 42 || name.charCodeAt(i) == 63 ||
			  	    name.charCodeAt(i) == 124 || name.charCodeAt(i) == 62 || name.charCodeAt(i) == 60 || name.charCodeAt(i) == 58  || name.charCodeAt(i) == 34) {
					this._displayErrorMessage(CommonMessages.ERROR_FOLDER_NAME+ ": * \\\ / | \  : < \" > ? ");
					return;
				}
			}
			var shared = this.nodeShared.checked;
			var parentFolderId = "";
			
			if (this.parentFolder.selectedIndex >= 0) {
				parentFolderId = this.parentFolder.options[this.parentFolder.selectedIndex].value;
			}
			
			var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			
			var owner = session.getAuthenticatedContributor();
			
			var self = this;
			
			var postExec = function() {
				self._controller.syncReportNodeChanged();				
			};
			
			this._postServiceExec = postExec;
			
			var srh = new ServiceResponseHandler(this, "_done", "_remoteError");
			var params = {
				name: name,
				parentFolderId: parentFolderId,
				projectAreaId: projectArea.itemId,
				ownerId: owner.itemId,
				shared: shared
			};
			ReportClient.postNewReportTreeFolder(srh, params);
		}
		else {
			// TODO: After creating a report folder, the user cannnot change any property.
			if (this.permissionsWidget != null) {
				this.permissionsWidget._updateAclEntries();	
			}
			this.saveButton.disabled = true;
		}
	},
	
	_done: function(treeNode) {
		this._msgFormatter.displayNoMessage();
		if (this._postServiceExec != null) {
			this._postServiceExec();
		}
		this._newTreeNode = treeNode;
		this._controller.reportTreeParser.expandFolder(this._selectedParentNodeId);
		this._controller._view._reportsNavSection._runAction("maximize");
		
		//moving to the edit view for folder
		this._controller.editReportNode(this._newTreeNode, null, this._postSave, this._postCancel);
	},
	
	_remoteError: function(error) {
		var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_CREATE_REPORT_FOLDER, [dojo.trim(this.folderName.value)]);
		this._msgFormatter.displayErrorMessage(msg, error);	
	},
	
	_newTabSelected: function(tab) {
		if (tab.dojoAttachPoint == "permissionContentPane") {
			if (this.permissionsWidget == null && this._createPermissionsTab == true) {
				this.permissionsWidget = new PermissionsWidget({
					repositoryItemId: this._newTreeNode.itemId,
					messageArea: this.messageArea,
					resourceType: this._newTreeNode.nodeType
				});
				this.permissionContentPane.appendChild(this.permissionsWidget.domNode);
			}
		}	
	},
	
	_disableTabs: function (e){
		e.preventDefault();
		e.stopPropagation();
	},
	
	_displayErrorMessage: function(errorMsg) {
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);	
	}
	
});

})();